@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2009-2013 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2009-2014 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -63,6 +63,5 @@ def connect(**kwargs):
     return Connection(**kwargs)
 
 def create_database(**kwargs):
-    if not 'page_size' in kwargs:
-        kwargs['page_size'] = 4096
+    kwargs['create_new'] = True
     return Connection(**kwargs)
