@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2009-2013 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2009-2014 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -26,7 +26,7 @@ except ImportError:
     from UserDict import DictMixin as Mapping
     HAS_MAPPING = False
 
-__version__ = '0.8.5'
+__version__ = '0.8.6'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
@@ -207,6 +207,7 @@ class Cursor:
                 self._fetch_records = self._fetch_generator(stmt_handle)
             else:
                 self._fetch_records = None
+            self._callproc_result = None
 
     def callproc(self, procname, params=[]):
         query = 'EXECUTE PROCEDURE ' + procname + ' ' + ','.join('?'*len(params))
@@ -454,9 +455,11 @@ class Connection(WireProtocol):
 
     def __init__(self, dsn=None, user=None, password=None, role=None, host=None,
                     database=None, charset=DEFAULT_CHARSET, port=3050,
-                    page_size=None, is_services=False, cloexec=False,
+                    page_size=4096, is_services=False, cloexec=False,
                     timeout=None, isolation_level=None, use_unicode=None,
-                    connect_version=2, use_srp=False, wire_crypt=False):
+                    connect_version=2, use_srp=True, wire_crypt=True,
+                    create_new=False):
+        self.db_handle = None
         if dsn:
             i = dsn.find(':')
             if i < 0:
@@ -503,7 +506,7 @@ class Connection(WireProtocol):
             self.sock = None
             e = sys.exc_info()[1]
             raise e
-        if self.page_size:                      # create database
+        if create_new:                      # create database
             self._op_create(self.page_size)
         elif self.is_services:                  # service api
             self._op_service_attach()
@@ -662,11 +665,12 @@ class Connection(WireProtocol):
             return
         if self._transaction:
             self._transaction.close()
-        if self.is_services:
-            self._op_service_detach()
-        else:
-            self._op_detach()
-        (h, oid, buf) = self._op_response()
+        if self.db_handle:
+            if self.is_services:
+                self._op_service_detach()
+            else:
+                self._op_detach()
+            (h, oid, buf) = self._op_response()
         self.sock.close()
         self.sock = None
         self.db_handle = None
