@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2013 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2014 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -265,7 +265,7 @@ def client_proof(user, password, salt, A, B, a):
     n1 = bytes2long(sha1(N))
     n2 = bytes2long(sha1(g))
     n1 = pow(n1, n2, N)
-    n2 = hash(user)
+    n2 = bytes2long(sha1(user))
     M = sha1(n1, n2, salt, A, B, K)
     if DEBUG_PRINT:
         print('client_proof:M=', binascii.b2a_hex(M), end='\n')
