@@ -1,5 +1,5 @@
 ##############################################################################
-# Copyright (c) 2009-2013 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2009-2014 Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
 # Licensed under the New BSD License
 # (http://www.freebsd.org/copyright/freebsd-license.html)
@@ -258,6 +258,9 @@ class WireProtocol(object):
             elif t == int:
                 v = bint_to_bytes(p, 4)
                 blr += bytes([8, 0])    # blr_long
+            elif t == float and p == float("inf"):
+                v = b'\x7f\x80\x00\x00'
+                blr += bytes([10])
             elif t == decimal.Decimal or t == float:
                 if t == float:
                     p = decimal.Decimal(str(p))
@@ -354,14 +357,12 @@ class WireProtocol(object):
         return r
 
     @wire_operation
-    def _op_connect(self, use_srp=False, wire_crypt=False):
+    def _op_connect(self, use_srp, wire_crypt):
         arch_type = 36
         min_arch_type = 0
         max_arch_type = 5
         protocol_version_understood_count = 4
         more_protocol = hex_to_bytes('ffff800b00000001000000000000000500000004ffff800c00000001000000000000000500000006ffff800d00000001000000000000000500000008')
-        if not use_srp and crypt is None:
-            self.connect_version = 2
         if self.connect_version ==2:
             max_arch_type = 3
             protocol_version_understood_count = 1
@@ -416,11 +417,9 @@ class WireProtocol(object):
 
         op_code = bytes_to_bint(b)
         b = self.recv_channel(12)
-        up = xdrlib.Unpacker(b)
-        protocol_version = up.unpack_int()
-        protocol_arch = up.unpack_int()
-        minimum_type =  up.unpack_int()
-        up.done()
+        protocol_version = b[:4]
+        protocol_arch = bytes_to_bint(b[4:8])
+        minimum_type =  bytes_to_bint(b[8:])
 
         if op_code == self.op_cond_accept or op_code == self.op_accept_data:
             read_length = 0
@@ -455,18 +454,15 @@ class WireProtocol(object):
                 ln = bytes_to_int(data[:2])
                 self.server_salt = data[2:ln+2]
                 self.server_public_key = hex_to_bytes(data[4+ln:])
-#                print('server_salt=', self.server_salt)
-#                print('server_public_key=', self.server_public_key)
                 self.server_public_key = srp.bytes2long(self.server_public_key)
 
                 self.client_proof, self.auth_key = srp.client_proof(
-                                            self.str_to_bytes(self.user),
-                                            self.str_to_bytes(self.password),
-                                            self.server_salt,
-                                            self.client_public_key,
-                                            self.server_public_key,
-                                            self.client_private_key)
-#                print('client_proof=', bytes_to_hex(self.client_proof))
+                                        self.str_to_bytes(self.user.upper()),
+                                        self.str_to_bytes(self.password),
+                                        self.server_salt,
+                                        self.client_public_key,
+                                        self.server_public_key,
+                                        self.client_private_key)
                 # send op_cont_auth
                 p = xdrlib.Packer()
                 p.pack_int(self.op_cont_auth)
@@ -482,8 +478,9 @@ class WireProtocol(object):
                 p.pack_int(self.op_crypt)
                 p.pack_string('Arc4')
                 p.pack_string('Symmetric')
-                self.sock.send(p.get_buffer() + b'\00\00\00')   # 3bytes padding
-                p.sock.set_translator(Arc4(self.auth_key), Arc4(self.auth_key))
+                self.sock.send(p.get_buffer())
+                self.sock.set_translator(
+                                    Arc4(self.auth_key), Arc4(self.auth_key))
                 (h, oid, buf) = self._op_response()
         else:
             assert op_code == self.op_accept
@@ -511,6 +508,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_drop_database(self):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_drop_database() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_drop_database)
         p.pack_int(self.db_handle)
@@ -533,9 +533,11 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_service_info(self, param, item, buffer_length=512):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_service_info() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_service_info)
-#        p.pack_int(self.svc_handle)
         p.pack_int(self.db_handle)
         p.pack_int(0)
         p.pack_bytes(param)
@@ -545,6 +547,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_service_start(self, param):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_service_start() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_service_start)
         p.pack_int(self.db_handle)
@@ -554,6 +559,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_service_detach(self):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_service_detach() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_service_detach)
         p.pack_int(self.db_handle)
@@ -561,6 +569,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_info_database(self, b):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_info_database() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_info_database)
         p.pack_int(self.db_handle)
@@ -571,6 +582,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_transaction(self, tpb):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_transaction() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_transaction)
         p.pack_int(self.db_handle)
@@ -607,6 +621,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_allocate_statement(self):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_allocate_statement() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_allocate_statement)
         p.pack_int(self.db_handle)
@@ -698,6 +715,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_exec_immediate(self, trans_handle, query):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_exec_immediate() Invalid db handle', None, None)
         desc_items = bytes([])
         p = xdrlib.Packer()
         p.pack_int(self.op_exec_immediate)
@@ -753,6 +773,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_detach(self):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_detach() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_detach)
         p.pack_int(self.db_handle)
@@ -814,6 +837,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_que_events(self, event_names, ast, args, event_id):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_que_events() Invalid db handle', None, None)
         params = bytes([1])
         for name, n in event_names.items():
             params += bytes([len(name)])
@@ -830,6 +856,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_cancel_events(self, event_id):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_cancel_events() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_cancel_events)
         p.pack_int(self.db_handle)
@@ -838,6 +867,9 @@ class WireProtocol(object):
 
     @wire_operation
     def _op_connect_request(self):
+        if self.db_handle is None:
+            raise OperationalError(
+                '_op_connect_request() Invalid db handle', None, None)
         p = xdrlib.Packer()
         p.pack_int(self.op_connect_request)
         p.pack_int(1)    # async
